/* Project 5 by David Taralla | 09-03-2010
*  ========================================
*  This program is used to create a mini
*  data base, associating a string to a key.
*  The user can get the data he entered back
*  if he refers to the key he associated with
*  this data.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include "Vector.h"


/******************************************************************************
*--------------------------------- New Types ---------------------------------*
******************************************************************************/
typedef struct {
    char key[21];
    char value[21];
} dataStructure;


/******************************************************************************
*--------------------------------- Functions ---------------------------------*
******************************************************************************/
/* ----------------------------------------------------------------------------
*  Description :
*  ==============
*  Delete unwanted carriage returns.
*  ----------------------------------------------------------------------------
*  Parameters :
*  =============
*  - Pointer leading to the string to modify.
*  ----------------------------------------------------------------------------
*  Return :
*  =========
*  - "\n" found   : 0 ;
*  - "\n" unfound : -1.
*  --------------------------------------------------------------------------*/
static int deleteNewLineChar(char *string) {
    char *ptr = strchr(string, '\n'); // Finding "\n"

    //If "\n" found, replacing it with "\0"
    if (ptr) {
        *ptr = 0;
        return 0;
    }
    
    // Not found
    return -1;
}


/* ----------------------------------------------------------------------------
*  Description :
*  ==============
*  Create and allocate a structure where are put the key and the value entered
*  by the user.
*  ----------------------------------------------------------------------------
*  Parameters :
*  =============
*  - Pointer leading to the key string ;
*  - Pointer leading to the value string.
*  ----------------------------------------------------------------------------
*  Return :
*  =========
*  - A pointer leading to the data structure containing the key and its value.
*  --------------------------------------------------------------------------*/
static dataStructure* putObjectInStruct(const char* enteredKey,
                                        const char* enteredValue) {
    dataStructure* newBlockPtr = (dataStructure*)malloc(sizeof(dataStructure));
    
    strncpy(newBlockPtr->key, enteredKey, 21);
    strncpy(newBlockPtr->value, enteredValue, 21);
    
    return newBlockPtr;
}


/* ----------------------------------------------------------------------------
*  Description :
*  ==============
*  Get the first index of an existing key.
*  ----------------------------------------------------------------------------
*  Parameters :
*  =============
*  - Pointer leading to the vector in which we are searching ;
*  - Pointer leading to the key string to find.
*  ----------------------------------------------------------------------------
*  Return :
*  =========
*  - Key found   : the first index of the key in the vector ;
*  - Key unfound : -1.
*  --------------------------------------------------------------------------*/
static int getKeyIndex(Vector* vector, char* key) {
    int vectorSize = vecSize(vector);
    for (int i = 0; i < vectorSize; i++) {
        if (strncmp(((dataStructure*)vecGet(vector, i))->key, key, 21) == 0)
            return i;
    }
    
    // Not found
    return -1;
}


/* ----------------------------------------------------------------------------
*  Description :
*  ==============
*  Free all manually allocated memory.
*  ----------------------------------------------------------------------------
*  Parameters :
*  =============
*  - Pointer leading to the vector containing elements to free.
*  --------------------------------------------------------------------------*/
static void freeElementsInVector(Vector* vector) {
    while (vecSize(vector) > 0) {
        dataStructure* v = (dataStructure*)vecRemove(vector, 0);
        if (v != NULL)
            free(v);
    }
}


/* ----------------------------------------------------------------------------
*  Description :
*  ==============
*  Empty the stdin buffer completely.
*  --------------------------------------------------------------------------*/
static void cleanStdinBuffer(void) {
    char c;    
    do {
        c = getchar();
    } while (c != '\n' && c != EOF);
}


/******************************************************************************
*------------------------------- Main program --------------------------------*
******************************************************************************/
int main() {
    const int MAX_SIZE = 21;
    
    Vector* dataVectorPtr = vecCreate();
    
    // -- ASKING PHASE --
    while (true) {
        // -- Gathering information --
        char newKey[MAX_SIZE];
        printf("New key: ");
        fgets(newKey, MAX_SIZE, stdin);
        
        if (newKey[0] == '\n') {
            // No key entered ; end of records
            break;
        }
        if (deleteNewLineChar(newKey) == -1)
            cleanStdinBuffer();
        
        char newValue[MAX_SIZE];
        printf("New value: ");
        fgets(newValue, MAX_SIZE, stdin);
        if (deleteNewLineChar(newValue) == -1)
            cleanStdinBuffer();
        
        // -- Recording information --
        int index = getKeyIndex(dataVectorPtr, newKey);
        if (index != -1) {
            // Key already exists at index [index]
            dataStructure* oldStruct =
                (dataStructure*)vecGet(dataVectorPtr, index);
            vecSet(dataVectorPtr, index, putObjectInStruct(newKey, newValue));
            free(oldStruct);
        }
        else {
            // Key doesn't exist - inserting at index 0
            vecInsert(dataVectorPtr, 0, putObjectInStruct(newKey, newValue));
        }
    }
    
    // -- PROVIDING PHASE --
    while (true) {
        // -- Gathering information --
        char oldKey[MAX_SIZE];
        printf("Old key: ");
        fgets(oldKey, MAX_SIZE, stdin);
        
        if (oldKey[0] == '\n') {
            // No key entered ; end of consultations
            break;
        }
        if (deleteNewLineChar(oldKey) == -1)
            cleanStdinBuffer();
        
        // -- Displaying result --
        int index = getKeyIndex(dataVectorPtr, oldKey);
        if (index != -1) {
            // Key found at [index]
            printf("Old value: %s\n",
                        ((dataStructure*)vecGet(dataVectorPtr, index))->value);
        }
        else {
            // Key doesn't exist - value is empty
            printf("Old value:\n");
        }
    }
    
    // -- CLEAN UP MEMORY --
    freeElementsInVector(dataVectorPtr);
    vecDelete(dataVectorPtr);
    return 0;
}
